angular.module('starter.services', [])
.factory('SessionService', function(){
	return {
		set:function(key,value){
			return localStorage.setItem(key,JSON.stringify(value));
		},
		get:function(key){
			var value = this.getString(key);
			if(typeof value == 'string' && value != ''){
				return JSON.parse(value);	
			}else{
				return false;
			}
			
		},
		getString: function(key){
			return localStorage.getItem(key);
		},
		destroy:function(key){
			return localStorage.removeItem(key);
		},
	};
})
.factory('OpalService', function($state, $ionicLoading, $http, $rootScope, $ionicPopup, SessionService, appConfig) {
	var $url 		= appConfig.DOMAIN_URL + '/opalestate-api/';
	return {
		imageUpload: $url + 'user/?task=avatar_upload&token=' + appConfig.TOKEN + '&key=' + appConfig.PUBLIC_KEY,
		sendRequest: function($type, $params, $isLoading) {
			var _url 	= $url + $type + '/?token=' + appConfig.TOKEN + '&key=' + appConfig.PUBLIC_KEY,
			loading 	= (typeof $isLoading == 'undefined') ? false : $isLoading;
			if(loading == true){
				$ionicLoading.show({
					template: '<ion-spinner></ion-spinner>'
				});
			}
			return $http({
				headers: {'Content-Type': 'application/x-www-form-urlencoded'},
				method: 'POST',
				url: _url,
				data: $params,
			})
			.then(function(response) {
				if(loading == true){
					$ionicLoading.hide();
				}
				return response.data;
			}, function error(response){
				$ionicLoading.hide();
				$ionicPopup.alert({
					title: 'Message',
					template: 'Please check your Internet connection and try again.'
				});
			})
			;
		}, // end sendRequest()
		checkLoggedIn: function(){
			if(SessionService.getString("is_login") !== null && SessionService.getString("is_login") === 'true')
				return true;
			else 
				return false;
		},
		getUserInfo: function(){
			var userInfo = SessionService.getString("userInfo");
			if(userInfo !== null){
				return JSON.parse(userInfo);
			}
			return false;
		},
		getMapStyle: function(){
			return [
			{
				"featureType":"administrative",
				"elementType":"labels.text.fill",
				"stylers":[
				{
					"color":"#444444"
				}
				]
			},
			{
				"featureType":"landscape",
				"elementType":"all",
				"stylers":[
				{
					"color":"#f2f2f2"
				}
				]
			},
			{
				"featureType":"poi",
				"elementType":"all",
				"stylers":[
				{
					"visibility":"off"
				}
				]
			},
			{
				"featureType":"road",
				"elementType":"all",
				"stylers":[
				{
					"saturation":-100
				},
				{
					"lightness":45
				}
				]
			},
			{
				"featureType":"road.highway",
				"elementType":"all",
				"stylers":[
				{
					"visibility":"simplified"
				}
				]
			},
			{
				"featureType":"road.arterial",
				"elementType":"labels.icon",
				"stylers":[
				{
					"visibility":"off"
				}
				]
			},
			{
				"featureType":"transit",
				"elementType":"all",
				"stylers":[
				{
					"visibility":"off"
				}
				]
			},
			{
				"featureType":"water",
				"elementType":"all",
				"stylers":[
				{
					"color":"#46bcec"
				},
				{
					"visibility":"on"
				}
				]
			}
			];
		}
	}
})
;