<?php 

class Opalestate_Action_Agents extends Opalestate_Abstract{

	public function action_contact(){
		$_POST['name'] 		= $this->get_param('name');		
		$_POST['email'] 	= $this->get_param('email');		
		$_POST['message'] 	= $this->get_param('message');		
		$_POST['post_id'] 	= $this->get_param('post_id');		
		$_POST['agent_id'] 	= $this->get_param('agent_id');		
		$_POST['author_id'] = $this->get_param('author_id');		
		return opalestate_process_send_email();
	}
	
	public function action_default(){
		$args = array(
			'post_type'        => 'opalestate_agent',
			'posts_per_page'   => $this->get_param('number'),
			'suppress_filters' => true,
			'paged'            => $this->get_param('paged')
		);
		return $this->_getQueryData($args, 'agents');
	}

	public function action_single(){
		$id = absint($this->get_param('id'));
		if($this->get_param('agent_type') == 'agent'){
			$agent_info = get_post( $this->get_param('id') );
			if($agent_info){
				return $this->_formatData( $agent_info );
			}
		}else{
			$user_info = get_user_by('id', $id);
			if($user_info){
				return $this->_formatAuthorData($user_info);
			}
		}
		
		return array();
	}

	/**
	 * Opalestaten a opalestate_agent post object, generate the data for the API output
	 *
	 * @since  1.1
	 *
	 * @param  object $agent_info The Download Post Object
	 *
	 * @return array                Array of post data to return back in the API
	 */
	public function _formatData( $agent_info ){
		$output = array();

		$output['info']['id']            = $agent_info->ID;
		$output['info']['number']        = $this->get_param('number');
		$output['info']['slug']          = $agent_info->post_name;
		$output['info']['title']         = $agent_info->post_title;
		$output['info']['create_date']   = $agent_info->post_date;
		$output['info']['modified_date'] = $agent_info->post_modified;
		$output['info']['status']        = $agent_info->post_status;
		$output['info']['link']          = html_entity_decode( $agent_info->guid );
		$output['info']['content']       = $agent_info->post_content;
		$output['info']['thumbnail']     = wp_get_attachment_url( get_post_thumbnail_id( $agent_info->ID ) );

		$agent = new OpalEstate_Agent( $agent_info->ID );
		$terms = wp_get_post_terms( $agent_info->ID, 'opalestate_agent_location' );
		
		$output['info']['featured']     = (int)$agent->is_featured();
		$output['info']['fax']		   	= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'fax', true );
		$output['info']['job']		   	= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'job', true );
		$output['info']['mobile']		= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'mobile', true );
		$output['info']['web']			= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'web', true );
		$output['info']['phone']		= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'phone', true );
		$output['info']['email']		= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'email', true );
		$output['info']['address']	   	= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'address', true );
		$output['info']['map']	   		= get_post_meta( $agent_info->ID, OPALESTATE_AGENT_PREFIX . 'map', true );
		$output['info']['location']	   	= $terms && !is_wp_error($terms) ? $terms : array();
	 
		$output['socials'] 				= $this->get_socials($agent_info->ID); 
		$output['levels'] 				= wp_get_post_terms( $agent_info->ID, 'opalestate_agent_level' );

		return apply_filters( 'opalestate_api_agent_single', $output );
	}

	public function _formatAuthorData($user_info){
		$output 	= array();
		$user_id	= $user_info->ID;

		$output['info']['title'] 	= $user_info->first_name . ' ' . $user_info->last_name;
		$output['info']['job'] 		= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'company', true );
		$output['info']['web'] 		= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'url', true );
		$output['info']['phone'] 	= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'phone', true );
		$output['info']['fax'] 		= '';
		$output['info']['mobile'] 	= '';
		$output['info']['email'] 	= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'email', true );
		$output['info']['content']	= $user_info->description; 
		$output['info']['thumbnail']= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'avatar', true );

		$output['socials']['facebook'] 	= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'facebook', true );
		$output['socials']['twitter'] 	= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'twitter', true );
		$output['socials']['pinterest'] = get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'pinterest', true );
		$output['socials']['google'] 	= get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'google', true );
		$output['socials']['instagram'] = get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'instagram', true );

		return apply_filters( 'opalestate_api_author_single', $output );
	}

	public function get_socials($id){
		$socials = array(
			'facebook',
			'twitter',
			'pinterest',
			'google',
			'instagram',
			'linkedIn',
		);
		
		$output = array();

		foreach( $socials as $social ){  

			$data = get_post_meta( $id, OPALESTATE_AGENT_PREFIX .  $social, true );
			if( $data && $data != "#" && !empty($data) ){ 
				$output[$social] = $data;
			}
		} 

		return $output;
	}

}