<?php 
class Opalestate_Action_Properties extends Opalestate_Abstract{

    private $favorites = array();

    public function __construct(){
        $user_id  = $this->get_param('user_id', 0);
        if($user_id){
            $this->favorites = get_user_meta( $this->get_param('user_id'),  'opalestate_user_favorite', true );
        }
    }

    public function action_delete_property(){
        $user_id = $this->get_param('user_id');
        $property_id = $this->get_param('property_id');

        // return array('status' => false, 'message' => __( 'An error occured when removing an item.', 'opalestate' ) );

        $is_allowed = Opalestate_Property::is_allowed_remove( $user_id, $property_id );

        if ( ! $is_allowed ) {
            return array('status' => false, 'message' =>  __( 'You are not allowed to access delete property.', 'opalestate-api' ) );
        }

        if ( wp_delete_post( $property_id ) ) {
            return array('status' => true, 'message' => __( 'Property has been successfully removed.', 'opalestate-api' ) );
        } else {
            return array('status' => false, 'message' => __( 'An error occured when removing an item.', 'opalestate-api' ) );
        }
    }

    public function action_toggle_featured(){
        $user_id        = intval( $this->get_param('user_id') );
        $property_id    = intval( $this->get_param('property_id') );

        $post = get_post( $property_id );
        return array(
            'post_author' => $post->post_author,
            'user_id'  => $user_id,
            'property_id' => $property_id,
            'count' => opalesate_get_user_featured_remaining_listing( $user_id ),
            );
        if( $post->post_author == $user_id ) {

            $check = opalesate_get_user_featured_remaining_listing( $user_id );
            if( $check ) {
                do_action( 'opalestate_toggle_featured_property_before', $user_id, $property_id );
                $is_featured = get_post_meta( $property_id, OPALESTATE_PROPERTY_PREFIX . 'featured', true );
                if($is_featured){
                    update_post_meta( $property_id, OPALESTATE_PROPERTY_PREFIX . 'featured', 0 );
                }else{
                    update_post_meta( $property_id, OPALESTATE_PROPERTY_PREFIX . 'featured', 1 );
                }
                return array( 'status' => true, 'msg' => __('Could not set this as featured','opalestate-api'), 'is_featured' => get_post_meta( $property_id, OPALESTATE_PROPERTY_PREFIX . 'featured', true ) );
            } 
        }  
        return array( 'status' => false, 'msg' => __('Could not set this as featured','opalestate-api'), 'is_featured' => get_post_meta( $property_id, OPALESTATE_PROPERTY_PREFIX . 'featured', true ) );
    }

    public function action_single_more_meta(){
        $property = opalesetate_property( $this->get_param('id') );

        return array(
            'data' => array(
                'amenities'     => $property->get_amenities(),
                'facilities'    => $property->get_facilities(),
            )
        );
    }

    public function action_favorites(){
        $paged  = $this->get_param('paged');
        $number = $this->get_param('number');
        $items  = (array)$this->favorites;

        $args = array(
            'post_type'         => 'opalestate_property',
            'paged'             => $paged,
            'posts_per_page'    => $number,
            'post__in'          => !empty($items) ? $items : array( 9999999 )
            );
        
        return $this->_getQueryData($args, 'properties');
    }

    public function action_search(){
        $properties = array();
        $paged 	= $this->get_param('paged');
        $number = $this->get_param('number');

        $_GET     = $this->get_params();
        $query    = OpalEstate_Search::get_search_results_query($this->per_page());

        if($paged != null){
            $query->set('paged', $paged);
        }

        if($number != null){
            $query->set('posts_per_page', $number);
        }

        if($query->have_posts()){
            $property_list = $query->get_posts();
            $i = 0;
            foreach ( $property_list as $property_info ) {
                $properties['properties'][$i] = $this->_formatData( $property_info );
                $i ++;
            }
        }

        return $properties;
    }

    public function action_search_near_location(){

        global $wpdb;
        $lat = $this->get_param('latitude');
        $lng = $this->get_param('longitude');

        $multiplier = 112.12; // use 69.0467669 if you want miles
        $distance   = $this->get_param('distance', 10); // kilometers or miles if 69.0467669

        $sql_query = "
        SELECT post.*
        FROM wp_posts AS post 
        INNER JOIN 
        (
        SELECT post_id, meta_value FROM wp_postmeta 
        WHERE meta_key = 'opalestate_ppt_map_latitude'
        ) meta1
        ON meta1.post_id = post.ID
        INNER JOIN 
        (
        SELECT post_id, meta_value FROM wp_postmeta 
        WHERE meta_key = 'opalestate_ppt_map_longitude'
        ) meta2
        ON meta2.post_id = post.ID
        WHERE
        post.post_type = 'opalestate_property'
        AND
        POW((meta1.meta_value - $lat), 2) + POW((meta2.meta_value - $lng), 2) < POW(($distance / $multiplier), 2)
        ";
        $results = $wpdb->get_results($sql_query, OBJECT);
        // return $results;

        if(count($results) > 0){
            $properties = array();
            foreach ($results as $property_info) {
                $properties['properties'][] = $this->_formatData( $property_info );
            }
            return array('status' => 1, 'data' => $properties);
        }else{
            return array('status' => 2);
        }

    }

    public function action_default(){
        $args = array(
            'post_type'        	=> 'opalestate_property',
            'posts_per_page'   	=> $this->per_page(),
            'suppress_filters' 	=> true,
            'paged'            	=> $this->get_param('paged'),
            );

        return $this->_getQueryData($args, 'properties');
    }

    public function action_user(){
        $args = array(
            'post_type'         => 'opalestate_property',
            'posts_per_page'    => $this->per_page(),
            'post_status'       => $this->get_param('status'),
            'author'            => $this->get_param('user_id'),
            'paged'             => $this->get_param('paged'),
            );
        return $this->_getQueryData($args, 'properties');
    }

    public function action_agent(){
        $args = array(
            'post_type'        	=> 'opalestate_property',
            'posts_per_page'   	=> $this->per_page(),
            'suppress_filters' 	=> true,
            'paged'            	=> $this->get_param('paged'),
            'meta_key' 			=> OPALESTATE_PROPERTY_PREFIX . 'agent',
            'meta_value'		=> $this->get_param('agent_id'),
            'meta_compare' 		=> '='
            );
        return $this->_getQueryData($args, 'properties');
    }

    public function action_featured(){
        $args = array(
            'post_type'        	=> 'opalestate_property',
            'posts_per_page'   	=> $this->per_page(),
            'suppress_filters' 	=> true,
            'paged'            	=> $this->get_param('paged'),
            'meta_key'			=> OPALESTATE_PROPERTY_PREFIX . 'featured',
            'meta_value'		    => 1,
            'meta_compare' 		=> '=',
            );
        return $this->_getQueryData($args, 'properties');
    }

    public function action_taxonomy(){
        $args = array(
            'post_type'         => 'opalestate_property',
            'posts_per_page'    => $this->per_page(),
            'suppress_filters'  => true,
            'paged'             => $this->get_param('paged'),
            'tax_query' => array(
                array(
                    'taxonomy'  => 'opalestate_status',
                    'field'     => 'slug',
                    'terms'     => $this->get_param('tax')
                    )
                )
            );
        // return array('test' => $this->get_param('tax'));
        return $this->_getQueryData($args, 'properties');
    }

    public function action_single(){
        $property_info = get_post( $this->get_param('id') );
        if($property_info){
            return $this->_formatData( $property_info );
        }
        return array();
    }

    public function action_comment(){
        $userid 	= $this->get_param('user_id');
        $params = array(
            'comment_post_ID' 		=> $this->get_param('id'),
            'comment_author_IP'		=> $_SERVER['REMOTE_ADDR'],
            'comment_agent' 		=> isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '',
            'comment_date'			=> current_time('mysql'),
            'comment_date_gmt' 		=> current_time('mysql', 1),
            'user_id' 				=> $this->get_param('user_id'),
            'comment_author' 		=> $this->get_param('user_name'),
            'comment_author_email' 	=> $this->get_param('user_email'),
            'comment_content' 		=> $this->get_param('content'),
            'comment_author_url'	=> $this->get_param('author_url'),
            'comment_parent' 		=> $this->get_param('parent', 0),
            'comment_type'			=> 'comment'
            );
        $params 	= wp_filter_comment($params);
        $response 	= $this->_check_comment($params);
        if($response['status']){
            $params['comment_approved'] = $response['approved'];
            $commentId 	= wp_insert_comment($params);
            $response['comment_id'] 	= $commentId;
        }
        return $response;
    }


    /**
    * check comment
    *
    * @param $commentdata
    * @return int|mixed|string|void
    * @throws Exception
    */
    private function _check_comment($commentdata){
        global $wpdb;
        $response = array(
            'status'	=> 1,
            );
        // Simple duplicate check
        // expected_slashed ($comment_post_ID, $comment_author, $comment_author_email, $comment_content)
        $dupe = $wpdb->prepare(
            "SELECT comment_ID FROM $wpdb->comments WHERE comment_post_ID = %d AND comment_parent = %s AND comment_approved != 'trash' AND ( comment_author = %s ",
            wp_unslash($commentdata['comment_post_ID']),
            wp_unslash($commentdata['comment_parent']),
            wp_unslash($commentdata['comment_author'])
            );
        if ($commentdata['comment_author_email']) {
            $dupe .= $wpdb->prepare(
                "OR comment_author_email = %s ",
                wp_unslash($commentdata['comment_author_email'])
                );
        }
        $dupe .= $wpdb->prepare(
            ") AND comment_content = %s LIMIT 1",
            wp_unslash($commentdata['comment_content'])
            );

        $dupe_id = $wpdb->get_var($dupe);

        /**
        * Filters the ID, if any, of the duplicate comment found when creating a new comment.
        *
        * Return an empty value from this filter to allow what WP considers a duplicate comment.
        *
        * @since 4.4.0
        *
        * @param int $dupe_id ID of the comment identified as a duplicate.
        * @param array $commentdata Data for the comment being created.
        */
        $dupe_id = apply_filters('duplicate_comment_id', $dupe_id, $commentdata);

        if ($dupe_id) {
        /**
        * Fires immediately after a duplicate comment is detected.
        *
        * @since 3.0.0
        *
        * @param array $commentdata Comment data.
        */
        $response['err_msg'] 	= __('Duplicate comment detected; it looks as though you’ve already said that!', 'opalestate-api');
        $response['status']  	= 0;
        return $response;
    }

    /**
    * Fires immediately before a comment is marked approved.
    *
    * Allows checking for comment flooding.
    *
    * @since 2.3.0
    *
    * @param string $comment_author_IP Comment author's IP address.
    * @param string $comment_author_email Comment author's email.
    * @param string $comment_date_gmt GMT date the comment was posted.
    */
    do_action(
        'check_comment_flood',
        $commentdata['comment_author_IP'],
        $commentdata['comment_author_email'],
        $commentdata['comment_date_gmt']
        );

    if (!empty($commentdata['user_id'])) {
        $user = get_userdata($commentdata['user_id']);
        $post_author = $wpdb->get_var($wpdb->prepare(
            "SELECT post_author FROM $wpdb->posts WHERE ID = %d LIMIT 1",
            $commentdata['comment_post_ID']
            ));
    }

    if (isset($user) && ($commentdata['user_id'] == $post_author || $user->has_cap('moderate_comments'))) {
    // The author and the admins get respect.
        $approved = 1;
    } else {
    // Everyone else's comments will be checked.
        if (check_comment(
            $commentdata['comment_author'],
            $commentdata['comment_author_email'],
            $commentdata['comment_author_url'],
            $commentdata['comment_content'],
            $commentdata['comment_author_IP'],
            $commentdata['comment_agent'],
            $commentdata['comment_type']
            )) {
            $approved = 1;
    } else {
        $approved = 0;
    }

    if (wp_blacklist_check(
        $commentdata['comment_author'],
        $commentdata['comment_author_email'],
        $commentdata['comment_author_url'],
        $commentdata['comment_content'],
        $commentdata['comment_author_IP'],
        $commentdata['comment_agent']
        )) {
        $approved = EMPTY_TRASH_DAYS ? 'trash' : 'spam';
}
}

        /**
         * Filter a comment's approval status before it is set.
         *
         * @since 2.1.0
         *
         * @param bool|string $approved The approval status. Accepts 1, 0, or 'spam'.
         * @param array $commentdata Comment data.
         */
        $response['approved'] 	= apply_filters('pre_comment_approved', $approved, $commentdata);

        return $response;
    }

	/**
	 * Opalestaten a opalestate_property post object, generate the data for the API output
	 *
	 * @since  1.1
	 *
	 * @param  object $property_info The Download Post Object
	 *
	 * @return array                Array of post data to return back in the API
	 */
	public function _formatData( $property_info ) {

		$property = array();



		$property['info']['id']            		= $property_info->ID;
		$property['info']['slug']          		= $property_info->post_name;
		$property['info']['title']         		= $property_info->post_title;
		$property['info']['create_date']   		= $property_info->post_date;
		$property['info']['modified_date'] 		= $property_info->post_modified;
		$property['info']['status']        		= $property_info->post_status;
		$property['info']['link']          		= html_entity_decode( $property_info->guid );
		$property['info']['content']       		= $property_info->post_content;
		$property['info']['thumbnail']     		= wp_get_attachment_url( get_post_thumbnail_id( $property_info->ID ) );
        $property['info']['comment_allowed']    = comments_open( $property_info->ID );
        $property['info']['featured'] 	        = get_post_meta($property_info->ID, OPALESTATE_PROPERTY_PREFIX . 'featured', true );

        $data 						 			= opalesetate_property( $property_info->ID );
        $gallery 					 			= $data->getGallery();
        $property['info']['gallery'] 			= isset($gallery[0]) && !empty($gallery[0]) ? $gallery[0]: array();
        $property['info']['price']              = opalestate_price_format( $data->get_price() );
        $property['info']['price_label'] 	 	= $data->get_price_label();
        $property['info']['map']	 			= $data->getMap();
        $property['info']['address']            = $data->get_address();
        $property['info']['agent_id']           = $data->get_metabox_value('agent');
        $property['info']['author_id'] 			= $property_info->post_author;

        $key = array_search( $property_info->ID, $this->favorites );
        if( $key != false || $key != '' ){
            $property['info']['in_favorites']   = true;
        }

        $property['meta_short']     = $data->get_meta_shortinfo();
        $property['meta_full']      = $data->get_meta_fullinfo();
        $property['status']         = $data->get_status();
        $property['locations']      = $data->get_locations();
        // $property['amenities']      = $data->get_amenities();
        $property['types']          = $data->get_types_tax();

		// if ( user_can( $this->user_id, 'view_opalestate_sensitive_data' ) || $this->override ) {

		// 	//Sensitive data here
		// 	do_action( 'opalestate_api_sensitive_data' );

		// }

        return apply_filters( 'opalestate_api_properties_property', $property );

    }

}