<?php 

class Opalestate_Action_User extends Opalestate_Abstract{

    public function action_update_meta(){
        $data       = json_decode($this->get_param('datajson'));
        $user_id    = $data->ID;
        $user_meta  = array('first_name', 'last_name', 'display_name', 'description');
        foreach ($user_meta as $meta_key) {
            update_user_meta( $user_id, $meta_key, $data->$meta_key);
        }
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'avatar',       $data->avatar );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'email',        $data->email );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'company',      $data->company );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'job',          $data->job );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'phone',        $data->phone );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'skype',        $data->skype );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'url',          $data->url );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'facebook',     $data->facebook );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'linkedin',     $data->linkedin );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'instagram',    $data->instagram );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'pinterest',    $data->pinterest );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'googleplus',   $data->googleplus );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'youtube',      $data->youtube );
        update_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'vimeo',        $data->vimeo );
        return array('success' => true);
    }

    public function action_register(){
        $response = array( 'success' => false );
        $username = $this->get_param('user_login', '');
        $email    = $this->get_param('user_email', '');
        if(username_exists($username)){
            $response['success']    = false;
            $response['message']    = __('Username is exists.', 'opalestate-api');
            return $response;
        }

        if(email_exists($email)){
            $response['success']    = false;
            $response['message']    = __('Email is exists.', 'opalestate-api');
            return $response;
        }

        $register   = wp_insert_user(array(
            'user_login'    => $username,
            'user_pass'     => $this->get_param('user_pass'),
            'user_email'    => $email,
        ));

        if (!is_wp_error($register)) {
            $response['success']    = true;
            $response['user']       = $this->_getUserById(get_user_by('id', $register));
        }else{
            $response['success']    = false;
            $response['message']    = false;
        }

        return $response;
    }

    public function action_update_password(){
        $data           = json_decode($this->get_param('datajson'));
        $userID         = $data->userId;
        $oldpassword    = $data->oldPass;
        $new_password   = $data->newPass;
        $user           = get_user_by( 'id', $userID );
        if( $user && wp_check_password( $oldpassword, $user->data->user_pass, $userID ) ) {
            wp_set_password( $new_password, $userID );
            return array('status' => true, 'message' => __( 'Password Updated', 'opalestate-api' ));
        } else {
            return array('status' => false, 'message' => __( 'Old password is not correct', 'opalestate-api' ));
        }
    }

    /**
     * Upload user avatar 
     */
    public function action_avatar_upload(){

        if (!function_exists( 'wp_handle_upload')){
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }
        if(!function_exists( 'wp_generate_attachment_metadata' )){
            include( ABSPATH . 'wp-admin/includes/image.php' );
        }
        $user_id = $this->get_param('user_id');
        opalestate_clean_attachments( $user_id ); 

        $submitted_file = $_FILES['file'];    
        $uploaded_image = wp_handle_upload( $submitted_file, array( 'test_form' => false ) );   //Handle PHP uploads in WordPress, sanitizing file names, checking extensions for mime type, and moving the file to the appropriate directory within the uploads directory.


        if ( isset( $uploaded_image['file'] ) ) {

            $file_name          =   basename( $submitted_file['name'] );
            $file_type          =   wp_check_filetype( $uploaded_image['file'] );   //Retrieve the file type from the file name.

            // Prepare an array of post data for the attachment.
            $attachment_details = array(
                'guid'           => $uploaded_image['url'],
                'post_mime_type' => $file_type['type'],
                'post_title'     => preg_replace( '/\.[^.]+$/', '', basename( $file_name ) ),
                'post_content'   => '',
                'post_status'    => 'inherit'
                );

            $attach_id      =   wp_insert_attachment( $attachment_details, $uploaded_image['file'] );       // This function inserts an attachment into the media library
            update_post_meta( $attach_id, '_pending_to_use', 1 );
            $attach_data    =   wp_generate_attachment_metadata( $attach_id, $uploaded_image['file'] );     // This function generates metadata for an image attachment. It also creates a thumbnail and other intermediate sizes of the image attachment based on the sizes defined
            wp_update_attachment_metadata( $attach_id, $attach_data );                                      // Update metadata for an attachment.


            $thumbnail_url = opalestate_get_upload_image_url( $attach_data );
            
            update_user_meta($user_id, OPALESTATE_USER_PROFILE_PREFIX . 'avatar', $thumbnail_url );
            update_user_meta($user_id, OPALESTATE_USER_PROFILE_PREFIX . 'avatar_id', $attach_id );


            $response = array(
                'success'           => true,
                'url'               => $thumbnail_url,
                'attachment_id'     => $attach_id,
                'user_id'           => $user_id,
                );

            
        } else {
            $response = array( 'success' => false, 'reason' => 'Image upload failed!' );
        }
        return $response;
    }

    public function action_payment(){
        return $response;
    }

    public function action_login_favorites(){
        $data = $this->_checkLogin();
        if($data){
            $user = wp_authenticate($data['user_login'], $data['user_pass']);
            if (!is_wp_error($user)) {
                $property_id    = absint( $this->get_param('propertyId') ); 
                $items          = (array)get_user_meta( $user->ID,  'opalestate_user_favorite', true );

                $key = array_search( $property_id, $items);
                if( $key != false || $key != '' ){
                    // unset($items[$key]);
                }else {
                    $items[] = $property_id;
                }
                update_user_meta( $user->ID, 'opalestate_user_favorite', $items );

                return array(
                    'msg'       => __('Added', 'opalestate-api'),
                    'favorites' => $items,
                    'data'      => $this->_getUserById($user),
                    );
            }
            return array('err_msg' => __("The email or password is invalid. Please try again.", 'opalestate-api') );
        }else{
            return array('err_msg' => __('Please check the username and password again.', 'opalestate-api' ) );
        }
    }
    
    public function action_toggle_favorites(){
        $user_id        = absint( $this->get_param('userId') );
        $property_id    = absint( $this->get_param('propertyId') ); 
        $items          = (array)get_user_meta( $user_id,  'opalestate_user_favorite', true );
        $key            = array_search( $property_id, $items);
        $action         = '';
        if( $key != false || $key != '' ){
            unset($items[$key]);
            $action     = 'remove';
        }else {
            $items[]    = $property_id;
            $action     = 'add';
        }

        update_user_meta( $user_id, 'opalestate_user_favorite', $items );
        return array('status' => $action, 'favorite' => get_user_meta($user_id, 'opalestate_user_favorite', true), 'userid' => $user_id);
    }


    public function action_login(){
        // return array(
        //     'username'  => $this->get_param('user_login'),
        //     'pass'      => $this->get_param('user_pass'),
        // );

        $data = $this->_checkLogin();
        if($data){
            $user = wp_authenticate($data['user_login'], $data['user_pass']);
            if (!is_wp_error($user)) {
                return $this->_getUserById($user);
            }
            return array('err_msg' => __("The email or password is invalid. Please try again.", 'opalestate-api') );
        }else{
            return array('err_msg' => __("Please check the username and password again.", 'opalestate-api') );
        }
    }


    private function _checkLogin(){
        $userLogin      = $this->get_param('user_login');
        $userPassword   = $this->get_param('user_pass');
        if ($userLogin && $userPassword) {
            return array(
                'user_login'    => $userLogin,
                'user_pass'     => $userPassword
                );
        }
        return false;
    }

    private function _getUserById($user){

        $return     = array();
        $user_id    = $user->ID;
        $return['info'] = array(
            'ID'            => $user->ID,
            'first_name'    => $user->first_name,
            'last_name'     => $user->last_name,
            'display_name'  => $user->display_name,
            'description'   => $user->description,
            'avatar'        => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'avatar',       true ),
            'email'         => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'email',        true ),
            'company'       => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'company',      true ),
            'job'           => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'job',          true ),
            'phone'         => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'phone',        true ),
            'skype'         => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'skype',        true ),
            'url'           => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'url',          true ),
            'facebook'      => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'facebook',     true ),
            'linkedin'      => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'linkedin',     true ),
            'instagram'     => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'instagram',    true ),
            'pinterest'     => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'pinterest',    true ),
            'googleplus'    => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'googleplus',   true ),
            'youtube'       => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'youtube',      true ),
            'vimeo'         => get_user_meta( $user_id, OPALESTATE_USER_PROFILE_PREFIX . 'vimeo',        true ),
        );

        if($this->is_membership){
            $payment_id = (int)get_user_meta( $user_id, OPALMEMBERSHIP_USER_PREFIX_.'payment_id', true );
            if($payment_id){
                $payment    = opalmembership_payment( $payment_id );
                $return['current_payment'] = array(
                    'id'                => $payment_id,
                    'purchased_date'    => $payment->payment_date,
                    'total_price'       => $payment->get_formatted_payment_total(),
                    'status'            => $payment->get_status_name(),
                    );
            }


            $package_id         = (int)get_user_meta( $user_id, OPALMEMBERSHIP_USER_PREFIX_.'package_id', true );
            if($package_id){
                $package        = opalmembership_package( $package_id );
                $unlimited      = get_post_meta( $package_id, OPALMEMBERSHIP_PACKAGES_PREFIX.'unlimited_listings', true );


                $return['current_package'] = array(
                    'id'            => $package_id,
                    'title'         => $package->post_title,
                    'title'         => $package->post_title,
                    'activation'    => get_user_meta( $user_id,     OPALMEMBERSHIP_USER_PREFIX_     .'package_activation', true ),
                    'expired'       => get_user_meta( $user_id,     OPALMEMBERSHIP_USER_PREFIX_     .'package_expired', true ),
                    'listings'      => get_post_meta( $package_id,  OPALMEMBERSHIP_PACKAGES_PREFIX  .'package_listings', true ),

                    'featured_listings'         => get_post_meta( $package_id, OPALMEMBERSHIP_PACKAGES_PREFIX.'package_featured_listings', true ),
                    'curent_featured_listings'  => get_user_meta($user_id, OPALMEMBERSHIP_USER_PREFIX_.'package_featured_listings',true),
                    'current_listings'          => get_user_meta($user_id, OPALMEMBERSHIP_USER_PREFIX_.'package_listings',true),
                    'unlimited_listings'        => $unlimited == 'on' ? 0 : 1,
                    );
            }
        }
            
        return $return;
    }


}